<?php

namespace Drupal\commerce_squad\Plugin\Commerce\PaymentGateway;

use Drupal\Core\Form\FormStateInterface;
use Drupal\commerce_order\Entity\OrderInterface;
use Drupal\commerce_payment\Exception\PaymentGatewayException;
use Drupal\commerce_payment\Plugin\Commerce\PaymentGateway\OffsitePaymentGatewayBase;
use Symfony\Component\HttpFoundation\Request;
use GuzzleHttp\ClientInterface;

/**
 * Provides the Squad Payment Gateway.
 *
 * @CommercePaymentGateway(
 *   id = "squad_standard",
 *   label = @Translation("Squad Payment Gateway"),
 *   display_label = @Translation("Squad Payment Gateway"),
 *   modes = {
 *     "sandbox" = @Translation("Sandbox"),
 *     "live" = @Translation("Live"),
 *   },
 *   forms = {
 *     "offsite-payment" = "Drupal\commerce_squad\PluginForm\SquadPaymentGatewayForm",
 *   },
 *   payment_method_types = {"credit_card"},
 *   credit_card_types = {
 *     "mastercard", "visa",
 *   },
 *   requires_billing_information = FALSE,
 * )
 */
class SquadPaymentGateway extends OffsitePaymentGatewayBase implements SquadPaymentGatewayInterface {
  
  /**
   * {@inheritdoc}
   */
  public function getMerchantId() {
    return $this->configuration['merchant_id'];
  }

  /**
   * {@inheritdoc}
   */
  public function getPrivateKey() {
    \Drupal::logger('commerce_squad')->debug('Configuration before checking private key in getPrivateKey: @config', ['@config' => print_r($this->configuration, TRUE)]);

    if (empty($this->configuration['private_key'])) {
        \Drupal::logger('commerce_squad')->error('Private key is not set in the configuration.');
        throw new PaymentGatewayException('Invalid or empty API key.');
    }

    \Drupal::logger('commerce_squad')->debug('Retrieved private key in getPrivateKey: @private_key', ['@private_key' => $this->configuration['private_key']]);

    return $this->configuration['private_key'];
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
        'mode' => 'sandbox',
        'currency' => 'NGN',
        'initiate_type' => 'inline',
        'pass_charge' => FALSE,
        'live_url' => 'https://api-d.squad.com',
        'sandbox_url' => 'https://sandbox-api-d.squad.com',
        'display_label' => 'Squad Payment Gateway',
        'collect_billing_information' => TRUE,
        'merchant_id' => '',
        'private_key' => '',
    ] + parent::defaultConfiguration();
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);

    // Add validation callbacks for the API key and merchant ID.
    $form['#element_validate'][] = [$this, 'validateConfigurationForm'];

    $form['mode'] = [
      '#type' => 'radios',
      '#title' => $this->t('Mode'),
      '#description' => $this->t('Select the mode for Squad (sandbox or live).'),
      '#default_value' => $this->configuration['mode'],
      '#options' => [
        'sandbox' => $this->t('Sandbox'),
        'live' => $this->t('Live'),
      ],
      '#required' => TRUE,
    ];

    $form['merchant_id'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Merchant ID'),
      '#description' => $this->t('Enter your Squad Merchant ID.'),
      '#default_value' => $this->configuration['merchant_id'],
      '#required' => TRUE,
    ];
    
    $form['private_key'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Private Key'),
      '#description' => $this->t('Enter your Squad API Private Key.'),
      '#default_value' => $this->configuration['private_key'],
      '#required' => TRUE,
    ];

    $form['pass_charge'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Pass Charge'),
      '#description' => $this->t('Whether to pass charges to the customer.'),
      '#default_value' => $this->configuration['pass_charge'],
    ];

    return $form;
  }

  public function validateConfigurationForm(array &$form, FormStateInterface $form_state) {
    $values = $form_state->getValue($form['#parents']);
    $private_key = $values['private_key'];
    $this->getPrivateKey();

    if (empty($private_key) || strpos($private_key, 'sandbox_sk_') !== 0) {
        $form_state->setErrorByName('private_key', $this->t('Invalid API key. Key must start with sandbox_sk_.'));
    }
  }

  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    parent::submitConfigurationForm($form, $form_state);
    $values = $form_state->getValue($form['#parents']);

    // Save the configuration settings.
    $this->configuration['mode'] = $values['mode'];
    $this->configuration['merchant_id'] = $values['merchant_id'];
    $this->configuration['private_key'] = $values['private_key'];
    $this->configuration['pass_charge'] = $values['pass_charge'];
  }


  /**
   * {@inheritdoc}
   */
  public function onReturn(OrderInterface $order, Request $request) {
    // Check the status of the transaction from the request.
    $transactionReference = $request->query->get('transaction_ref');


    // Call the verifyTransaction method to get the transaction status.
    $transactionStatus = $this->verifyTransaction($this->configuration, $transactionReference);

    // Handle different transaction statuses accordingly.
    switch ($transactionStatus) {
        case 'success':
            // Transaction was successful, update the order status.
            $order->getState()->applyTransitionById('complete');
            $order->save();
            // $this->messenger->addMessage($this->t('Payment successful.'));
            break;

        case 'failed':
            // Transaction failed, update the order status.
            $order->getState()->applyTransitionById('cancel');
            $order->save();
            // $this->messenger->addError($this->t('Payment failed.'));
            break;

        case 'abandoned':
            // Transaction was abandoned, update the order status.
            $order->getState()->applyTransitionById('cancel');
            $order->save();
            // $this->messenger->addMessage($this->t('Payment abandoned.'));
            break;

        case 'pending':
            // Transaction is pending, update the order status if needed.
            // Optionally, inform the user about the pending transaction.
            // $this->messenger->addMessage($this->t('Payment pending.'));
            break;

        default:
            // Handle unknown transaction status.
            // $this->messenger->addError($this->t('Unknown transaction status.'));
            break;
    }
  }

  /**
   * {@inheritdoc}
   */
  public function onCancel(OrderInterface $order, Request $request) {
    // Update the order status to canceled.
    $order->getState()->applyTransitionById('cancel');
    $order->save();
  }

  /**
   * Verifies the status of a transaction.
   *
   * @param string $transactionReference
   *   The transaction reference.
   *
   * @return string
   *   The transaction status.
   *
   * @throws \Drupal\commerce_payment\Exception\PaymentGatewayException
   *   Thrown when the verification fails.
   */
  public function verifyTransaction($transactionReference) {
    $mode = $this->configuration['mode'];
    $base_url = $mode === 'live' ? $this->configuration['live_url'] : $this->configuration['sandbox_url'];

    // Initialize Guzzle HTTP client.
    $client = new \GuzzleHttp\Client();

    try {
        // Make a request to the Squad API to verify the transaction status.
        $response = $client->get("$base_url/transaction/verify/$transactionReference", [
            'headers' => [
                'Authorization' => 'Bearer ' . $this->getPrivateKey(),
            ],
        ]);

        $statusCode = $response->getStatusCode();
        $data = json_decode($response->getBody()->getContents(), TRUE);

        if ($statusCode === 200 && isset($data['data']['transaction_status'])) {
            return $data['data']['transaction_status'];
        } else {
            throw new \Exception('Failed to verify transaction. Status Code: ' . $statusCode);
        }
    } catch (\Exception $e) {
        \Drupal::logger('commerce_squad')->error('Transaction verification failed: @message', ['@message' => $e->getMessage()]);
        throw new PaymentGatewayException('Failed to verify transaction: ' . $e->getMessage());
    }
  }
}